@@ -1,30 +1,29 @@
 use std::path::{Path, PathBuf};
 use std::process::Command;
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, bail};
+use tracing::{error, warn};
 
 use super::install_bins;
-use super::shell;
-use crate::currentprocess::varsource::VarSource;
-use crate::process;
-use crate::utils::utils;
-use crate::utils::Notification;
+use super::shell::{self, Posix, UnixShell};
+use crate::process::Process;
+use crate::utils::{self, Notification};
 
 // If the user is trying to install with sudo, on some systems this will
 // result in writing root-owned files to the user's home directory, because
 // sudo is configured not to change $HOME. Don't let that bogosity happen.
-pub(crate) fn do_anti_sudo_check(no_prompt: bool) -> Result<utils::ExitCode> {
-    pub(crate) fn home_mismatch() -> (bool, PathBuf, PathBuf) {
+pub(crate) fn do_anti_sudo_check(no_prompt: bool, process: &Process) -> Result<utils::ExitCode> {
+    pub(crate) fn home_mismatch(process: &Process) -> (bool, PathBuf, PathBuf) {
         let fallback = || (false, PathBuf::new(), PathBuf::new());
         // test runner should set this, nothing else
-        if process()
+        if process
             .var_os("RUSTUP_INIT_SKIP_SUDO_CHECK")
-            .map_or(false, |s| s == "yes")
+            .is_some_and(|s| s == "yes")
         {
             return fallback();
         }
 
-        match (utils::home_dir_from_passwd(), process().var_os("HOME")) {
+        match (utils::home_dir_from_passwd(), process.var_os("HOME")) {
             (Some(pw), Some(eh)) if eh != pw => return (true, PathBuf::from(eh), pw),
             (None, _) => warn!("getpwuid_r: couldn't get user data"),
             _ => {}
@@ -32,14 +31,14 @@ pub(crate) fn do_anti_sudo_check(no_prompt: bool) -> Result<utils::ExitCode> {
         fallback()
     }
 
-    match home_mismatch() {
+    match home_mismatch(process) {
         (false, _, _) => {}
         (true, env_home, euid_home) => {
-            err!("$HOME differs from euid-obtained home directory: you may be using sudo");
-            err!("$HOME directory: {}", env_home.display());
-            err!("euid-obtained home directory: {}", euid_home.display());
+            error!("$HOME differs from euid-obtained home directory: you may be using sudo");
+            error!("$HOME directory: {}", env_home.display());
+            error!("euid-obtained home directory: {}", euid_home.display());
             if !no_prompt {
-                err!("if this is what you want, restart the installation with `-y'");
+                error!("if this is what you want, restart the installation with `-y'");
                 return Ok(utils::ExitCode(1));
             }
         }
@@ -48,17 +47,17 @@ pub(crate) fn do_anti_sudo_check(no_prompt: bool) -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
-pub(crate) fn delete_rustup_and_cargo_home() -> Result<()> {
-    let cargo_home = utils::cargo_home()?;
+pub(crate) fn delete_rustup_and_cargo_home(process: &Process) -> Result<()> {
+    let cargo_home = process.cargo_home()?;
     utils::remove_dir("cargo_home", &cargo_home, &|_: Notification<'_>| ())
 }
 
-pub(crate) fn do_remove_from_path() -> Result<()> {
-    for sh in shell::get_available_shells() {
-        let source_bytes = format!("{}\n", sh.source_string()?).into_bytes();
+pub(crate) fn do_remove_from_path(process: &Process) -> Result<()> {
+    for sh in shell::get_available_shells(process) {
+        let source_bytes = format!("{}\n", sh.source_string(process)?).into_bytes();
 
         // Check more files for cleanup than normally are updated.
-        for rc in sh.rcfiles().iter().filter(|rc| rc.is_file()) {
+        for rc in sh.rcfiles(process).iter().filter(|rc| rc.is_file()) {
             let file = utils::read_file("rcfile", rc)?;
             let file_bytes = file.into_bytes();
             // FIXME: This is whitespace sensitive where it should not be.
@@ -75,17 +74,17 @@ pub(crate) fn do_remove_from_path() -> Result<()> {
         }
     }
 
-    remove_legacy_paths()?;
+    remove_legacy_paths(process)?;
 
     Ok(())
 }
 
-pub(crate) fn do_add_to_path() -> Result<()> {
-    for sh in shell::get_available_shells() {
-        let source_cmd = sh.source_string()?;
+pub(crate) fn do_add_to_path(process: &Process) -> Result<()> {
+    for sh in shell::get_available_shells(process) {
+        let source_cmd = sh.source_string(process)?;
         let source_cmd_with_newline = format!("\n{}", &source_cmd);
 
-        for rc in sh.update_rcs() {
+        for rc in sh.update_rcs(process) {
             let cmd_to_write = match utils::read_file("rcfile", &rc) {
                 Ok(contents) if contents.contains(&source_cmd) => continue,
                 Ok(contents) if !contents.ends_with('\n') => &source_cmd_with_newline,
@@ -104,19 +103,19 @@ pub(crate) fn do_add_to_path() -> Result<()> {
         }
     }
 
-    remove_legacy_paths()?;
+    remove_legacy_paths(process)?;
 
     Ok(())
 }
 
-pub(crate) fn do_write_env_files() -> Result<()> {
+pub(crate) fn do_write_env_files(process: &Process) -> Result<()> {
     let mut written = vec![];
 
-    for sh in shell::get_available_shells() {
+    for sh in shell::get_available_shells(process) {
         let script = sh.env_script();
         // Only write each possible script once.
         if !written.contains(&script) {
-            script.write()?;
+            sh.write_script(&script, process)?;
             written.push(script);
         }
     }
@@ -124,14 +123,6 @@ pub(crate) fn do_write_env_files() -> Result<()> {
     Ok(())
 }
 
-pub(crate) fn do_add_to_programs() -> Result<()> {
-    Ok(())
-}
-
-pub(crate) fn do_remove_from_programs() -> Result<()> {
-    Ok(())
-}
-
 /// Tell the upgrader to replace the rustup bins, then delete
 /// itself.
 pub(crate) fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
@@ -150,15 +141,15 @@ pub(crate) fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
 /// This function is as the final step of a self-upgrade. It replaces
 /// `CARGO_HOME`/bin/rustup with the running exe, and updates the
 /// links to it.
-pub(crate) fn self_replace() -> Result<utils::ExitCode> {
-    install_bins()?;
+pub(crate) fn self_replace(process: &Process) -> Result<utils::ExitCode> {
+    install_bins(process)?;
 
     Ok(utils::ExitCode(0))
 }
 
-fn remove_legacy_source_command(source_cmd: String) -> Result<()> {
+fn remove_legacy_source_command(source_cmd: String, process: &Process) -> Result<()> {
     let cmd_bytes = source_cmd.into_bytes();
-    for rc in shell::legacy_paths().filter(|rc| rc.is_file()) {
+    for rc in shell::legacy_paths(process).filter(|rc| rc.is_file()) {
         let file = utils::read_file("rcfile", &rc)?;
         let file_bytes = file.into_bytes();
         // FIXME: This is whitespace sensitive where it should not be.
@@ -176,18 +167,24 @@ fn remove_legacy_source_command(source_cmd: String) -> Result<()> {
     Ok(())
 }
 
-fn remove_legacy_paths() -> Result<()> {
+fn remove_legacy_paths(process: &Process) -> Result<()> {
     // Before the work to support more kinds of shells, which was released in
     // version 1.23.0 of Rustup, we always inserted this line instead, which is
     // now considered legacy
-    remove_legacy_source_command(format!(
-        "export PATH=\"{}/bin:$PATH\"\n",
-        shell::cargo_home_str()?
-    ))?;
+    remove_legacy_source_command(
+        format!(
+            "export PATH=\"{}/bin:$PATH\"\n",
+            Posix.cargo_home_str(process)?
+        ),
+        process,
+    )?;
     // Unfortunately in 1.23, we accidentally used `source` rather than `.`
     // which, while widely supported, isn't actually POSIX, so we also
     // clean that up here.  This issue was filed as #2623.
-    remove_legacy_source_command(format!("source \"{}/env\"\n", shell::cargo_home_str()?))?;
+    remove_legacy_source_command(
+        format!("source \"{}/env\"\n", Posix.cargo_home_str(process)?),
+        process,
+    )?;
 
     Ok(())
 }
