@@ -1,26 +1,39 @@
-#![allow(clippy::box_default)]
+#![allow(
+    clippy::box_default,
+    clippy::print_stdout,
+    clippy::print_stderr,
+    clippy::dbg_macro
+)]
 //! Test support module; public to permit use from integration tests.
 
-pub mod mock;
-
 use std::collections::HashMap;
 use std::env;
 use std::ffi::OsStr;
 use std::fmt;
 use std::fs;
-use std::io;
+use std::fs::File;
+use std::io::{self, Read};
 use std::path::{Path, PathBuf};
 use std::process::Command;
 
 #[cfg(test)]
 use anyhow::Result;
+use sha2::{Digest, Sha256};
 
-pub use crate::cli::self_update::test::{with_saved_global_state, with_saved_path};
-use crate::currentprocess;
-use crate::dist::dist::TargetTriple;
+use crate::dist::TargetTriple;
+use crate::process::TestProcess;
 
 #[cfg(windows)]
-pub use crate::cli::self_update::test::{get_path, with_saved_reg_value};
+pub use crate::cli::self_update::{RegistryGuard, RegistryValueId, USER_PATH, get_path};
+
+mod clitools;
+pub use clitools::{
+    CliTestContext, Config, SanitizedOutput, Scenario, SelfUpdateTestContext, output_release_file,
+    print_command, print_indented,
+};
+pub(crate) mod dist;
+pub(crate) mod mock;
+pub use mock::{MockComponentBuilder, MockFile, MockInstallerBuilder};
 
 // Things that can have environment variables applied to them.
 pub trait Env {
@@ -113,22 +126,22 @@ fn tempdir_in_with_prefix<P: AsRef<Path>>(path: P, prefix: &str) -> io::Result<P
 /// ... perhaps this is so that the test data we have is only exercised on known
 /// triples?
 ///
-/// NOTE: This *cannot* be called within a currentprocess context as it creates
+/// NOTE: This *cannot* be called within a process context as it creates
 /// its own context on Windows hosts. This is partly by chance but also partly
 /// deliberate: If you need the host triple, or to call for_host(), you can do
-/// so outside of calls to run() or unit test code that runs in a currentprocess
+/// so outside of calls to run() or unit test code that runs in a process
 /// context.
 ///
 /// IF it becomes very hard to workaround that, then we can either make a second
-/// this_host_triple that doesn't make its own currentprocess or use
-/// TargetTriple::from_host() from within the currentprocess context as needed.
+/// this_host_triple that doesn't make its own process or use
+/// TargetTriple::from_host() from within the process context as needed.
 pub fn this_host_triple() -> String {
     if cfg!(target_os = "windows") {
         // For windows, this host may be different to the target: we may be
         // building with i686 toolchain, but on an x86_64 host, so run the
         // actual detection logic and trust it.
-        let tp = currentprocess::TestProcess::default();
-        return currentprocess::with(tp.into(), || TargetTriple::from_host().unwrap().to_string());
+        let tp = TestProcess::default();
+        return TargetTriple::from_host(&tp.process).unwrap().to_string();
     }
     let arch = if cfg!(target_arch = "x86") {
         "i686"
@@ -140,6 +153,10 @@ pub fn this_host_triple() -> String {
         "aarch64"
     } else if cfg!(target_arch = "loongarch64") {
         "loongarch64"
+    } else if cfg!(target_arch = "powerpc64") && cfg!(target_endian = "little") {
+        "powerpc64le"
+    } else if cfg!(target_arch = "s390x") {
+        "s390x"
     } else {
         unimplemented!()
     };
@@ -170,8 +187,8 @@ pub fn this_host_triple() -> String {
 // Format a string with this host triple.
 #[macro_export]
 macro_rules! for_host {
-    ($s: expr) => {
-        &format!($s, $crate::test::this_host_triple())
+    ($s:tt $($arg:tt)*) => {
+        &format!($s, $crate::test::this_host_triple() $($arg)*)
     };
 }
 
@@ -224,85 +241,80 @@ where
     f(&rustup_home)
 }
 
-#[cfg(feature = "otel")]
-use once_cell::sync::Lazy;
+pub mod topical_doc_data {
+    use std::collections::HashSet;
+    use std::path::PathBuf;
 
-/// A tokio runtime for the sync tests, permitting the use of tracing. This is
-/// never shutdown, instead it is just dropped at end of process.
-#[cfg(feature = "otel")]
-static TRACE_RUNTIME: Lazy<tokio::runtime::Runtime> =
-    Lazy::new(|| tokio::runtime::Runtime::new().unwrap());
-/// A tracer for the tests.
-#[cfg(feature = "otel")]
-static TRACER: Lazy<opentelemetry_sdk::trace::Tracer> = Lazy::new(|| {
-    use std::time::Duration;
+    // Paths are written as a string in the UNIX format to make it easy
+    // to maintain.
+    static TEST_CASES: &[(&[&str], &str)] = &[
+        (&["core"], "core/index.html"),
+        (&["core::arch"], "core/arch/index.html"),
+        (&["fn"], "std/keyword.fn.html"),
+        (&["keyword:fn"], "std/keyword.fn.html"),
+        (&["primitive:fn"], "std/primitive.fn.html"),
+        (&["macro:file!"], "std/macro.file!.html"),
+        (&["macro:file"], "std/macro.file.html"),
+        (&["std::fs"], "std/fs/index.html"),
+        (&["std::fs::read_dir"], "std/fs/fn.read_dir.html"),
+        (&["std::io::Bytes"], "std/io/struct.Bytes.html"),
+        (&["std::iter::Sum"], "std/iter/trait.Sum.html"),
+        (&["std::io::error::Result"], "std/io/error/type.Result.html"),
+        (&["usize"], "std/primitive.usize.html"),
+        (&["eprintln"], "std/macro.eprintln.html"),
+        (&["alloc::format"], "alloc/macro.format.html"),
+        (&["std::mem::MaybeUninit"], "std/mem/union.MaybeUninit.html"),
+        (&["--rustc", "lints"], "rustc/lints/index.html"),
+        (&["--rustdoc", "lints"], "rustdoc/lints.html"),
+        (
+            &["lints::broken_intra_doc_links", "--rustdoc"],
+            "rustdoc/lints.html",
+        ),
+    ];
 
-    use opentelemetry::{global, KeyValue};
-    use opentelemetry_otlp::WithExportConfig;
-    use opentelemetry_sdk::{
-        propagation::TraceContextPropagator,
-        trace::{self, Sampler},
-        Resource,
-    };
-    use tokio::runtime::Handle;
-    use tracing_subscriber::{layer::SubscriberExt, EnvFilter, Registry};
+    fn repath(origin: &str) -> String {
+        // Add doc prefix and rewrite string paths for the current platform
+        let with_prefix = "share/doc/rust/html/".to_owned() + origin;
+        let splitted = with_prefix.split('/');
+        let repathed = splitted.fold(PathBuf::new(), |acc, e| acc.join(e));
+        repathed.into_os_string().into_string().unwrap()
+    }
 
-    // Use the current runtime, or the sync test runtime otherwise.
-    let handle = match Handle::try_current() {
-        Ok(handle) => handle,
-        Err(_) => TRACE_RUNTIME.handle().clone(),
-    };
-    let _guard = handle.enter();
+    pub fn test_cases<'a>() -> impl Iterator<Item = (&'a [&'a str], String)> {
+        TEST_CASES.iter().map(|(args, path)| (*args, repath(path)))
+    }
 
-    let tracer = opentelemetry_otlp::new_pipeline()
-        .tracing()
-        .with_exporter(
-            opentelemetry_otlp::new_exporter()
-                .tonic()
-                .with_timeout(Duration::from_secs(3)),
-        )
-        .with_trace_config(
-            trace::config()
-                .with_sampler(Sampler::AlwaysOn)
-                .with_resource(Resource::new(vec![KeyValue::new("service.name", "rustup")])),
-        )
-        .install_batch(opentelemetry_sdk::runtime::Tokio)
-        .unwrap();
-
-    global::set_text_map_propagator(TraceContextPropagator::new());
-    let env_filter = EnvFilter::try_from_default_env().unwrap_or(EnvFilter::new("INFO"));
-    let telemetry = tracing_opentelemetry::layer().with_tracer(tracer.clone());
-    let subscriber = Registry::default().with(env_filter).with(telemetry);
-    tracing::subscriber::set_global_default(subscriber).unwrap();
-    tracer
-});
-
-pub fn before_test() {
-    #[cfg(feature = "otel")]
-    {
-        Lazy::force(&TRACER);
+    pub fn unique_paths() -> impl Iterator<Item = String> {
+        // Hashset used to test uniqueness of values through insert method.
+        let mut unique_paths = HashSet::new();
+        TEST_CASES
+            .iter()
+            .filter(move |(_, p)| unique_paths.insert(p))
+            .map(|(_, p)| repath(p))
     }
 }
 
-pub async fn before_test_async() {
-    #[cfg(feature = "otel")]
-    {
-        Lazy::force(&TRACER);
-    }
+pub fn calc_hash(src: &Path) -> String {
+    let mut buf = Vec::new();
+    File::open(src).unwrap().read_to_end(&mut buf).unwrap();
+    let mut hasher = Sha256::new();
+    hasher.update(buf);
+    format!("{:x}", hasher.finalize())
 }
 
-pub fn after_test() {
-    #[cfg(feature = "otel")]
-    {
-        let handle = TRACE_RUNTIME.handle();
-        let _guard = handle.enter();
-        TRACER.provider().map(|p| p.force_flush());
-    }
+pub fn create_hash(src: &Path, dst: &Path) -> String {
+    let hex = calc_hash(src);
+    let src_file = src.file_name().unwrap();
+    let file_contents = format!("{} *{}\n", hex, src_file.to_string_lossy());
+    dist::write_file(dst, &file_contents);
+    hex
 }
 
-pub async fn after_test_async() {
-    #[cfg(feature = "otel")]
-    {
-        TRACER.provider().map(|p| p.force_flush());
-    }
-}
+pub static CROSS_ARCH1: &str = "x86_64-unknown-linux-musl";
+pub static CROSS_ARCH2: &str = "arm-linux-androideabi";
+
+// Architecture for testing 'multi-host' installation.
+#[cfg(target_pointer_width = "64")]
+pub static MULTI_ARCH1: &str = "i686-unknown-linux-gnu";
+#[cfg(not(target_pointer_width = "64"))]
+pub static MULTI_ARCH1: &str = "x86_64-unknown-linux-gnu";
